@@ -53,6 +53,10 @@ class Transaction:
     def UnspentCoins(self):
         return GetUnspentCoins(self)
 
+    @property
+    def Witnesses(self):
+        return GetWitnesses(self)
+
 
 def GetTXHash(transaction):
     """
@@ -112,3 +116,15 @@ def GetUnspentCoins(transaction):
 
     """
     pass
+
+
+def GetWitnesses(transaction):
+    """
+
+    Args:
+        transaction:
+
+    Returns:
+
+    """
+    pass
