@@ -7,8 +7,10 @@ from neo.SmartContract.SmartContractEvent import SmartContractEvent, NotifyEvent
 from neo.Settings import settings
 from neo.Prompt.Utils import parse_param
 from neo.Core.FunctionCode import FunctionCode
-from neocore.Fixed8 import Fixed8
-from boa_test.example.demo.nex.token import *
+from neo.Core.Fixed8 import Fixed8
+from boa_test.example.demo.token.crowdsale import *
+from boa_test.example.demo.token.nep5 import *
+from neo.Blockchain import GetBlockchain
 
 import shutil
 import os
@@ -23,6 +25,8 @@ class TestContract(BoaFixtureTest):
     dispatched_events = []
     dispatched_logs = []
 
+    snapshot = None
+
     @classmethod
     def tearDownClass(cls):
         super(BoaFixtureTest, cls).tearDownClass()
@@ -49,6 +53,7 @@ class TestContract(BoaFixtureTest):
             cls.dispatched_logs.append(evt)
         events.on(SmartContractEvent.RUNTIME_NOTIFY, on_notif)
         events.on(SmartContractEvent.RUNTIME_LOG, on_log)
+        cls.snapshot = GetBlockchain()._db.createSnapshot()
 
     def test_ICOTemplate_1(self):
 
@@ -56,47 +61,49 @@ class TestContract(BoaFixtureTest):
         out = output.write()
 #        print(output.to_s())
 
-        tx, results, total_ops, engine = TestBuild(out, ['name', '[]'], self.GetWallet1(), '0705', '05')
+        snapshot = self.snapshot
+
+        tx, results, total_ops, engine = TestBuild(out, ['name', '[]'], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetString(), TOKEN_NAME)
 
-        tx, results, total_ops, engine = TestBuild(out, ['symbol', '[]'], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['symbol', '[]'], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetString(), TOKEN_SYMBOL)
 
-        tx, results, total_ops, engine = TestBuild(out, ['decimals', '[]'], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['decimals', '[]'], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBigInteger(), TOKEN_DECIMALS)
 
-        tx, results, total_ops, engine = TestBuild(out, ['totalSupply', '[]'], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['totalSupply', '[]'], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
-        self.assertEqual(results[0].GetBigInteger(), 0)
+        self.assertEqual(results[0].GetBigInteger(), TOKEN_TOTAL_SUPPLY)
 
-        tx, results, total_ops, engine = TestBuild(out, ['nonexistentmethod', '[]'], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['nonexistentmethod', '[]'], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetString(), 'unknown operation')
 
         # deploy with wallet 2 should fail CheckWitness
-        tx, results, total_ops, engine = TestBuild(out, ['deploy', '[]'], self.GetWallet2(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['deploy', '[]'], self.GetWallet2(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), False)
 
-        tx, results, total_ops, engine = TestBuild(out, ['deploy', '[]'], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['deploy', '[]'], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), True)
 
         # second time, it should already be deployed and return false
-        tx, results, total_ops, engine = TestBuild(out, ['deploy', '[]'], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['deploy', '[]'], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), False)
 
-        # now total supply should be equal to the initial owner amount
-        tx, results, total_ops, engine = TestBuild(out, ['totalSupply', '[]'], self.GetWallet1(), '0705', '05')
+        # now circulation should be equal to the initial owner amount
+        tx, results, total_ops, engine = TestBuild(out, ['circulation', '[]'], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBigInteger(), TOKEN_INITIAL_AMOUNT)
 
         # now the owner should have a balance of the TOKEN_INITIAL_AMOUNT
-        tx, results, total_ops, engine = TestBuild(out, ['balanceOf', parse_param([bytearray(TOKEN_OWNER)])], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['balanceOf', parse_param([bytearray(TOKEN_OWNER)])], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBigInteger(), TOKEN_INITIAL_AMOUNT)
 
@@ -105,12 +112,13 @@ class TestContract(BoaFixtureTest):
         output = Compiler.instance().load('%s/boa_test/example/demo/ICO_Template.py' % TestContract.dirname).default
         out = output.write()
 
+        snapshot = self.snapshot
         # now transfer tokens to wallet 2
 
         TestContract.dispatched_events = []
 
         test_transfer_amount = 2400000001
-        tx, results, total_ops, engine = TestBuild(out, ['transfer', parse_param([bytearray(TOKEN_OWNER), self.wallet_2_script_hash.Data, test_transfer_amount])], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['transfer', parse_param([bytearray(TOKEN_OWNER), self.wallet_2_script_hash.Data, test_transfer_amount])], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), True)
 
@@ -122,37 +130,37 @@ class TestContract(BoaFixtureTest):
         self.assertEqual(evt.amount, test_transfer_amount)
 
         # now get balance of wallet 2
-        tx, results, total_ops, engine = TestBuild(out, ['balanceOf', parse_param([self.wallet_2_script_hash.Data])], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['balanceOf', parse_param([self.wallet_2_script_hash.Data])], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBigInteger(), test_transfer_amount)
 
         # now the owner should have less
-        tx, results, total_ops, engine = TestBuild(out, ['balanceOf', parse_param([bytearray(TOKEN_OWNER)])], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['balanceOf', parse_param([bytearray(TOKEN_OWNER)])], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBigInteger(), TOKEN_INITIAL_AMOUNT - test_transfer_amount)
 
         # now this transfer should fail
-        tx, results, total_ops, engine = TestBuild(out, ['transfer', parse_param([bytearray(TOKEN_OWNER), self.wallet_2_script_hash.Data, TOKEN_INITIAL_AMOUNT])], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['transfer', parse_param([bytearray(TOKEN_OWNER), self.wallet_2_script_hash.Data, TOKEN_INITIAL_AMOUNT])], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), False)
 
         # this transfer should fail because it is not signed by the 'from' address
-        tx, results, total_ops, engine = TestBuild(out, ['transfer', parse_param([bytearray(TOKEN_OWNER), self.wallet_2_script_hash.Data, 10000])], self.GetWallet3(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['transfer', parse_param([bytearray(TOKEN_OWNER), self.wallet_2_script_hash.Data, 10000])], self.GetWallet3(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), False)
 
         # now this transfer should fail, this is from address with no tokens
-        tx, results, total_ops, engine = TestBuild(out, ['transfer', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data, 1000])], self.GetWallet3(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['transfer', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data, 1000])], self.GetWallet3(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), False)
 
         # get balance of bad data
-        tx, results, total_ops, engine = TestBuild(out, ['balanceOf', parse_param(['abc'])], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['balanceOf', parse_param(['abc'])], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBigInteger(), 0)
 
         # get balance no params
-        tx, results, total_ops, engine = TestBuild(out, ['balanceOf', parse_param([])], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['balanceOf', parse_param([])], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), False)
 
@@ -160,30 +168,33 @@ class TestContract(BoaFixtureTest):
 
         output = Compiler.instance().load('%s/boa_test/example/demo/ICO_Template.py' % TestContract.dirname).default
         out = output.write()
-        print(output.to_s())
+        string_ouput = output.to_s()
+        self.assertGreater(len(string_ouput), 0)
         # now transfer tokens to wallet 2
 
         TestContract.dispatched_events = []
 
+        snapshot = self.snapshot
+
         # test mint tokens without being kyc verified
-        tx, results, total_ops, engine = TestBuild(out, ['mintTokens', '[]', '--attach-neo=10'], self.GetWallet3(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['mintTokens', '[]', '--attach-neo=10'], self.GetWallet3(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), False)
 
         # Try to register as a non owner
-        tx, results, total_ops, engine = TestBuild(out, ['crowdsale_register', parse_param([self.wallet_3_script_hash.Data])], self.GetWallet3(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['crowdsale_register', parse_param([self.wallet_3_script_hash.Data])], self.GetWallet3(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), False)
 
         # Get status of non registered address
-        tx, results, total_ops, engine = TestBuild(out, ['crowdsale_status', parse_param([self.wallet_3_script_hash.Data])], self.GetWallet3(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['crowdsale_status', parse_param([self.wallet_3_script_hash.Data])], self.GetWallet3(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), False)
 
         TestContract.dispatched_events = []
 
         # register an address
-        tx, results, total_ops, engine = TestBuild(out, ['crowdsale_register', parse_param([self.wallet_3_script_hash.Data])], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['crowdsale_register', parse_param([self.wallet_3_script_hash.Data])], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBigInteger(), 1)
 
@@ -193,12 +204,12 @@ class TestContract(BoaFixtureTest):
         self.assertEqual(evt.event_payload.Value[0].Value, b'kyc_registration')
 
         # register 2 addresses at once
-        tx, results, total_ops, engine = TestBuild(out, ['crowdsale_register', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data])], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['crowdsale_register', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data])], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBigInteger(), 2)
 
         # now check reg status
-        tx, results, total_ops, engine = TestBuild(out, ['crowdsale_status', parse_param([self.wallet_3_script_hash.Data])], self.GetWallet3(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['crowdsale_status', parse_param([self.wallet_3_script_hash.Data])], self.GetWallet3(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), True)
 
@@ -207,8 +218,10 @@ class TestContract(BoaFixtureTest):
         output = Compiler.instance().load('%s/boa_test/example/demo/ICO_Template.py' % TestContract.dirname).default
         out = output.write()
 
+        snapshot = self.snapshot
+
         # test mint tokens without being kyc verified
-        tx, results, total_ops, engine = TestBuild(out, ['get_attachments', '[]', '--attach-neo=10'], self.GetWallet3(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['get_attachments', '[]', '--attach-neo=10'], self.GetWallet3(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         attachments = results[0].GetArray()
         self.assertEqual(len(attachments), 4)
@@ -220,7 +233,7 @@ class TestContract(BoaFixtureTest):
         self.assertEqual(attachments[2].GetBigInteger(), Fixed8.FromDecimal(10).value)
         self.assertEqual(attachments[3].GetBigInteger(), 0)
 
-        tx, results, total_ops, engine = TestBuild(out, ['get_attachments', '[]'], self.GetWallet3(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['get_attachments', '[]'], self.GetWallet3(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         attachments = results[0].GetArray()
         self.assertEqual(len(attachments), 4)
@@ -229,7 +242,7 @@ class TestContract(BoaFixtureTest):
         self.assertEqual(attachments[2].GetBigInteger(), 0)
         self.assertEqual(attachments[3].GetBigInteger(), 0)
 
-        tx, results, total_ops, engine = TestBuild(out, ['get_attachments', '[]', '--attach-neo=3', '--attach-gas=3.12'], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['get_attachments', '[]', '--attach-neo=3', '--attach-gas=3.12'], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         attachments = results[0].GetArray()
         self.assertEqual(len(attachments), 4)
@@ -242,15 +255,17 @@ class TestContract(BoaFixtureTest):
         output = Compiler.instance().load('%s/boa_test/example/demo/ICO_Template.py' % TestContract.dirname).default
         out = output.write()
 
+        snapshot = self.snapshot
+
         # register an address
-        tx, results, total_ops, engine = TestBuild(out, ['crowdsale_register', parse_param([self.wallet_3_script_hash.Data])], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['crowdsale_register', parse_param([self.wallet_3_script_hash.Data])], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBigInteger(), 1)
 
         TestContract.dispatched_events = []
 
         # test mint tokens, this should return true
-        tx, results, total_ops, engine = TestBuild(out, ['mintTokens', '[]', '--attach-neo=10'], self.GetWallet3(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['mintTokens', '[]', '--attach-neo=10'], self.GetWallet3(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), True)
 
@@ -262,17 +277,17 @@ class TestContract(BoaFixtureTest):
         self.assertEqual(evt.addr_to, self.wallet_3_script_hash)
 
         # test mint tokens again, this should be false since you can't do it twice
-        tx, results, total_ops, engine = TestBuild(out, ['mintTokens', '[]', '--attach-neo=10'], self.GetWallet3(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['mintTokens', '[]', '--attach-neo=10'], self.GetWallet3(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), False)
 
         # now the minter should have a balance
-        tx, results, total_ops, engine = TestBuild(out, ['balanceOf', parse_param([self.wallet_3_script_hash.Data])], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['balanceOf', parse_param([self.wallet_3_script_hash.Data])], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBigInteger(), 10 * TOKENS_PER_NEO)
 
         # now the total circulation should be bigger
-        tx, results, total_ops, engine = TestBuild(out, ['totalSupply', '[]'], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['circulation', '[]'], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBigInteger(), (10 * TOKENS_PER_NEO) + TOKEN_INITIAL_AMOUNT)
 
@@ -281,30 +296,32 @@ class TestContract(BoaFixtureTest):
         output = Compiler.instance().load('%s/boa_test/example/demo/ICO_Template.py' % TestContract.dirname).default
         out = output.write()
 
+        snapshot = self.snapshot
+
         # tranfer_from, approve, allowance
-        tx, results, total_ops, engine = TestBuild(out, ['allowance', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data])], self.GetWallet2(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['allowance', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data])], self.GetWallet2(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBigInteger(), 0)
 
         # try to transfer from
-        tx, results, total_ops, engine = TestBuild(out, ['transferFrom', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data, 10000])], self.GetWallet2(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['transferFrom', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data, 10000])], self.GetWallet2(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), False)
 
         # try to approve from someone not yourself
-        tx, results, total_ops, engine = TestBuild(out, ['approve', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data, 10000])], self.GetWallet2(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['approve', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data, 10000])], self.GetWallet2(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBigInteger(), 0)
 
         # try to approve more than you have
-        tx, results, total_ops, engine = TestBuild(out, ['approve', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data, TOKEN_INITIAL_AMOUNT])], self.GetWallet3(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['approve', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data, TOKEN_INITIAL_AMOUNT])], self.GetWallet3(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBigInteger(), 0)
 
         TestContract.dispatched_events = []
 
         # approve should work
-        tx, results, total_ops, engine = TestBuild(out, ['approve', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data, 1234])], self.GetWallet3(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['approve', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data, 1234])], self.GetWallet3(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), True)
 
@@ -316,43 +333,43 @@ class TestContract(BoaFixtureTest):
         self.assertEqual(evt.amount, 1234)
 
         # check allowance
-        tx, results, total_ops, engine = TestBuild(out, ['allowance', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data])], self.GetWallet2(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['allowance', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data])], self.GetWallet2(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBigInteger(), 1234)
 
         # approve should not be additive, it should overwrite previous approvals
-        tx, results, total_ops, engine = TestBuild(out, ['approve', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data, 133234])], self.GetWallet3(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['approve', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data, 133234])], self.GetWallet3(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), True)
 
-        tx, results, total_ops, engine = TestBuild(out, ['allowance', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data])], self.GetWallet2(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['allowance', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data])], self.GetWallet2(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBigInteger(), 133234)
 
         # now you can transfer from
-        tx, results, total_ops, engine = TestBuild(out, ['transferFrom', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data, 10000])], self.GetWallet2(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['transferFrom', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data, 10000])], self.GetWallet2(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), True)
 
         # now the recevier should have a balance
         # it is equal to 10000 plus test_transfer_amount = 2400000001
 
-        tx, results, total_ops, engine = TestBuild(out, ['balanceOf', parse_param([self.wallet_2_script_hash.Data])], self.GetWallet1(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['balanceOf', parse_param([self.wallet_2_script_hash.Data])], self.GetWallet1(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBigInteger(), 10000 + 2400000001)
 
         # now the allowance should be less
-        tx, results, total_ops, engine = TestBuild(out, ['allowance', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data])], self.GetWallet2(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['allowance', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data])], self.GetWallet2(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBigInteger(), 133234 - 10000)
 
         # try to transfer too much, even with approval
-        tx, results, total_ops, engine = TestBuild(out, ['transferFrom', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data, 14440000])], self.GetWallet2(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['transferFrom', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data, 14440000])], self.GetWallet2(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), False)
 
         # cant approve negative amounts
-        tx, results, total_ops, engine = TestBuild(out, ['approve', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data, -1000])], self.GetWallet3(), '0705', '05')
+        tx, results, total_ops, engine = TestBuild(out, ['approve', parse_param([self.wallet_3_script_hash.Data, self.wallet_2_script_hash.Data, -1000])], self.GetWallet3(), '0705', '05', snapshot=snapshot)
         self.assertEqual(len(results), 1)
         self.assertEqual(results[0].GetBoolean(), False)
 
